'use strict';

exports.__esModule = true;
exports.I18N = exports.Label = exports.default = undefined;
exports.getTerm = getTerm;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Dictionaries = {};

var I18N = function (_Component) {
  _inherits(I18N, _Component);

  function I18N(props) {
    _classCallCheck(this, I18N);

    var _this = _possibleConstructorReturn(this, _Component.call(this, props));

    _this.state = {
      __DICS: Dictionaries
    };
    _this.instanceID = Math.ceil(Math.random() * 999999) + new Date().getTime();
    _this.setDictionary = _this.setDictionary.bind(_this);
    return _this;
  }

  I18N.prototype.getVal = function getVal(key, values) {
    return key;
  };

  I18N.prototype.setDictionary = function setDictionary(props) {
    var _this2 = this;

    var dictionaries = props.use;
    dictionaries = Array.isArray(dictionaries) ? dictionaries : [dictionaries];
    dictionaries.forEach(function (dict) {
      if (!dict.id) {
        console.error('Failed using dictionary: ', dict);
        throw new Error('Your dictionary must have an ID');
      }
      var dictId = dict.id + ':' + (dict.version || 1);
      var lang = props.lang || 'en-US';
      var theDict = Dictionaries[_this2.instanceID];
      var instanceID = _this2.instanceID;
      var langBase = lang.split('-')[0] || '';

      if (!theDict) {
        Dictionaries[instanceID] = {
          lang: lang,
          fallbackLang: props.fallbackLang || langBase,
          baseLang: langBase,
          books: {}
        };
      }
      if (theDict && theDict.lang !== lang) {
        Dictionaries[_this2.instanceID].lang = lang;
        if (Dictionaries[_this2.instanceID].fallbackLang !== 'none') {
          Dictionaries[_this2.instanceID].fallbackLang = langBase;
        }
        Dictionaries[_this2.instanceID].baseLang = langBase;
      }
      Dictionaries[_this2.instanceID].books[dictId] = dict;
    });
  };

  I18N.prototype.componentDidMount = function componentDidMount() {
    this.setDictionary(this.props);
  };

  I18N.prototype.componentWillReceiveProps = function componentWillReceiveProps(newProps) {
    this.setDictionary(newProps);
  };

  I18N.prototype.render = function render() {
    return _react2.default.createElement(
      'div',
      {
        className: this.props.className,
        'data-i18nId': this.instanceID,
        key: this.props.lang },
      this.props.children
    );
  };

  return I18N;
}(_react.Component);

exports.default = I18N;

var Label = exports.Label = function (_Component2) {
  _inherits(Label, _Component2);

  function Label(props) {
    _classCallCheck(this, Label);

    var _this3 = _possibleConstructorReturn(this, _Component2.call(this, props));

    _this3.state = {
      __DICS: Dictionaries
    };
    _this3.setValues = _this3.setValues.bind(_this3);
    _this3.updateValues = _this3.updateValues.bind(_this3);
    _this3.done = _this3.done.bind(_this3);
    _this3._getTerm = _this3._getTerm.bind(_this3);
    return _this3;
  }

  Label.prototype.done = function done(term) {
    this.setState({
      term: term
    });
  };

  Label.prototype.applyVariables = function applyVariables(term, props) {
    var _this4 = this;

    var match = term.match(/\$\{[a-zA-Z0-9_]+\}/g);
    if (!match) {
      return term;
    }
    match.forEach(function (found) {
      found = found.slice(2, -1); // removed ${ and }
      var value = props[found] || '';
      if (Array.isArray(value)) {
        if (value.length === 1) {
          value = value[0];
        } else {
          if (value.length === 0) {
            value = '';
          } else {
            var joints = _this4._getTerm('joints');
            if (joints) {
              value = Array.from(value);
              var last = value.pop();
              value = value.join(joints[0] || ', ');
              value += (props.jointType === 'none' ? joints[0] : props.jointType === 'or' ? joints[2] : joints[1]) || ', ';
              value += last;
            } else {
              value = value.join(', ');
            }
          }
        }
      }
      term = term.replace(new RegExp('\\$\{' + found + '\}', 'g'), value);
    });
    return term;
  };

  Label.prototype._getTerm = function _getTerm(lookingFor) {
    var dicts = Dictionaries[this.state.i18nid];
    var term = null;

    term = getTerm(lookingFor, dicts.books, dicts.lang, dicts.fallbackLang, dicts.baseLang);
    return term;
  };

  Label.prototype.updateValues = function updateValues(props) {
    var lookingFor = props.term;

    var term = this._getTerm(lookingFor);
    if (Array.isArray(term)) {
      term = term[props.val === 1 ? 0 : 1];
    }
    if (!term) {
      if (Dictionaries[this.state.i18nid].fallbackLang === 'none') {
        term = '';
      } else {
        term = lookingFor.split(/(?=[A-Z])/);
        term = term[0] + ' ' + term.slice(1).join(' ').toLowerCase();
        console.warn('Missing term in dictionaries: ', lookingFor);
      }
    } else {
      term = this.applyVariables(term, props);
    }
    this.done(term);
  };

  Label.prototype.setValues = function setValues(props) {
    var _this5 = this;

    if (!this.state.i18nid) {
      var originalNode = _reactDom2.default.findDOMNode(this).parentElement;
      var node = originalNode;
      while (!node.dataset.i18nid) {
        node = node.parentElement;
        if (node.tagName === 'BODY' || node.tagName === 'HTML') {
          console.error('Could not find the I18N component as parent of: ', originalNode);
          throw new Error('Could not find the I18N component on the ascending tree!');
        }
      }
      var obj = {
        i18nid: node.dataset.i18nid
      };
      if (Dictionaries[node.dataset.i18nid]) {
        obj.lang = Dictionaries[node.dataset.i18nid].lang;
      }
      this.setState(obj, function (_) {
        return _this5.updateValues(props);
      });
    } else {
      this.updateValues(props);
    }
  };

  Label.prototype.componentDidMount = function componentDidMount() {
    this.setValues(this.props);
  };
  // shouldComponentUpdate (props, state) {
  //   return true
  // }


  Label.prototype.componentWillReceiveProps = function componentWillReceiveProps(newProps) {
    this.setValues(newProps);
  };

  Label.prototype.render = function render(props) {
    return _react2.default.createElement(
      'span',
      { className: this.props.className, key: this.state.lang },
      this.state.term
    );
  };

  return Label;
}(_react.Component);

// (lookingFor, dicts.books, dicts.lang, dicts.fallbackLang, dicts.baseLang)


function getTerm(lookingFor, books, lang, fallbackLang, baseLang) {
  var term = null;
  for (var dict in books) {
    if (term) {
      return term;
    }
    var curLangs = books[dict].langs;
    term = (curLangs[lang] || {})[lookingFor] || (curLangs[fallbackLang] || {})[lookingFor] || (curLangs[baseLang] || {})[lookingFor] || (curLangs.world || {})[lookingFor];
  }
  return term;
}

exports.I18N = I18N;